/*
 * Copyright (c) 2016 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Wed Jul  6 2016
 */
#include "acdynamixel.h"

#include "dynamixel_c_types.h"
#include "codels.h"


/* --- Task cmd --------------------------------------------------------- */


/* --- Activity set_pid ------------------------------------------------- */

/** Codel dx_set_pid_start of activity set_pid.
 *
 * Triggered by dynamixel_start.
 * Yields to dynamixel_ether.
 * Throws dynamixel_e_sys.
 */
genom_event
dx_set_pid_start(dynamixel_channel_s **chan, uint8_t id, double kp,
                 double ki, double kd, genom_context self)
{
  int d, i, p;

  if ((*chan)->fd < 0) {
    errno = ENOENT;
    return dx_e_sys_error(NULL, self);
  }

  if (id < 1 || id > 0xfc) return dynamixel_ether;

  d = kd * 1000. / 4;
  i = ki * 2048. / 1000;
  p = kp * 8.;

  if (d < 0 || d > 0xff ||
      i < 0 || i > 0xff ||
      p < 0 || p > 0xff) {
    errno = ERANGE;
    return dx_e_sys_error(NULL, self);
  }

  while (dx_recv_msg(*chan, true) == 1); /* flush buffer */

  dx_send_msg(*chan, id, DX_WRITE, 3, DX_D_GAIN, d, i, p);
  return dynamixel_ether;
}
