/*
 * Copyright (c) 2016 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Wed Jul  6 2016
 */
#include "acdynamixel.h"

#include <math.h>

#include "dynamixel_c_types.h"


/* --- Attribute set_position ------------------------------------------- */

/** Validation codel dx_set_position_validate of attribute set_position.
 *
 * Returns genom_ok.
 * Throws .
 */
genom_event
dx_set_position_validate(sequence8_double *position,
                         sequence8_double *velocity,
                         sequence8_double *effort, genom_context self)
{
  size_t l;

  /* discard trailing nans */
  l = position->_length;
  while(l > 0 && isnan(position->_buffer[l-1])) l--;
  position->_length = l;

  /* reset other input */
  velocity->_length = 0;
  effort->_length = 0;

  return genom_ok;
}


/* --- Attribute set_velocity ------------------------------------------- */

/** Validation codel dx_set_velocity_validate of attribute set_velocity.
 *
 * Returns genom_ok.
 * Throws .
 */
genom_event
dx_set_velocity_validate(sequence8_double *position,
                         sequence8_double *velocity,
                         sequence8_double *effort, genom_context self)
{
  size_t l;

  /* discard trailing nans */
  l = velocity->_length;
  while(l > 0 && isnan(velocity->_buffer[l-1])) l--;
  velocity->_length = l;

  /* reset other input */
  position->_length = 0;
  effort->_length = 0;

  return genom_ok;
}


/* --- Attribute set_effort --------------------------------------------- */

/** Validation codel dx_set_effort_validate of attribute set_effort.
 *
 * Returns genom_ok.
 * Throws .
 */
genom_event
dx_set_effort_validate(sequence8_double *position,
                       sequence8_double *velocity,
                       sequence8_double *effort, genom_context self)
{
  size_t l;

  /* discard trailing nans */
  l = effort->_length;
  while(l > 0 && isnan(effort->_buffer[l-1])) l--;
  effort->_length = l;

  /* reset other input */
  position->_length = 0;
  velocity->_length = 0;

  return genom_ok;
}
